{{
*************************************
* TvFavorites.spin is a demo of the
* SFE Nokia Knockoff LCD gui*.spin widgets
* Copyright (c) 2009 John Steven Denson
* See MIT license terms below
*************************************
}}

{{

TvFavorites:

  A TV Channel Navigation Control using Propeller and SFE Nokiaoff LCD

  My TV has too many channels now and there are only a few I care to see.
  This design will hopefully solve that problem. My wife needs one too :)

  User Interface -- 2 Buttons and an LCD:

    Normal Mode:
    
        Left  Button - Select ... scrolls through channel number and title list
        Right Button - Ok for channel number ... Edit for title.
        Both  Buttons - Go to "Settings Mode"
         
    Settings Mode:

        Left  Button  - Select Menu Task
        Right Button  - Go to Task
        Both  Buttons - not assigned
         
    Menu:
     
        Add Channel    - Use remote to set channel number / title.
        Delete Channel - Select channel and Delete.
        Edit Channel   - Rename channel with remote.
        Move Channel   - Select channel with Ok, move to new place, insert with OK.
        Train Device   - Teach device how to read and replay numbers, etc....
        Normal Mode    - Go back to normal mode.
         
    Add Channel:

        Two text boxes provided for number and title.
        Left Button Select - Right Button OK 
        Select moves from one box to another.
        Enter chars from remote with 1-9; use * to go LEFT , > to go RIGHT.
        Right Button OK to finish.
        
    Delete Channel:

        Show channel list
        Left Button Select - Right Button OK 
        Select channel and delete with OK
                 
    Edit Channel:

        Show channel list - screen 1, Show edit - screen 2
        Left Button Select - Right Button OK
        Select channel to edit and press OK to start edit
        Next screen ... Show editing text box.
        Enter chars from remote with 1-9; use * to LEFT, > to advance.
        Right Button OK to finish.

    Move Channel:

        Show channel list - screen 1, Show channel list - screen 2
        Left Button Select - Right Button OK 
        Select channel to move and press OK to choose channel
        Next screen ... Select new position and press OK
        
    Train Device:

        [TvFavorites Training]
         [1] [2] [3]
         [4] [5] [6]
         [7] [8] [9]
         [<] [0] [>]
           [enter]
        [Press remote 1 key]
        
        Show Keypad buttons and mode text boxes.
        Left Button Select - Right Button OK
        Text box 1 shows "Train" or "Test"
        Text box 2 starts with "Press remote 1 button."
        Select highlights border of chosen keypad button.
        User enters corresponding remote key button to train.
        User enters key button again to test.
        Keypad button flashes for train and test.
        When ok, button turns gray and border highlight moves to next button.
        Use select to re-train. - pending.
        Right Button OK to finish. - pending.  

    Normal Mode:

        Left Button Select - Right Button OK
        Right Button OK to exit settings        

                         
  Copyright (c) 2009 by John Steven Denson

}}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  '_clkmode = RCFAST

  CR    = $d
{
  ' display pins
  SCS   = 14 '27
  SCK   = 13 '26
  SDIO  = 12 '25
  SRST  = 11 '24
}
  SCS   = 4
  SCK   = 5
  SDIO  = 6
  SRST  = 7

  ' control buttons  
  LBUT  = 0 '22 
  RBUT  = LBUT+1 ' must be contiguous

  ' Primary EEPROM
  EEPROM1_SCL = 28
  EEPROM1_DAT = 29


  ' IR connections  
  IRIN  = 16
  'IROUT = 0

  CHANNELS = def#CHANNELS
  IRKEYS   = 12 ' cant use def#IRKEYS

  NO_BUTTONS    = 3
  LEFT_BUTTON   = 1
  RIGHT_BUTTON  = 2
  BOTH_BUTTONS  = 0
  
OBJ
  lcd: "spiasm-lcd-65k"
  def: "TvDefines"
  rem: "TvRemote"
  key: "TvKeys"
  ia : "itoatoi"
  sf : "snprintf"
  
  eep: "EEPROM"

  btSel         : "gui_button"
  btOk          : "gui_button"
  lbListUm      : "gui_listbox"
  lbListSm      : "gui_listbox"
  laInput[2]    : "gui_label"
  tbInput[2]    : "gui_textbox"
  tbStatus      : "gui_textbox"
  tbPrompt      : "gui_textbox"
  tbDebug       : "gui_textbox"
  btKey[IRKEYS] : "gui_button"

VAR
  byte state, screen
  long scog
  long stack[$28]
  long irvalid
  byte irbuf[rem#IRBUFLEN]
'  byte polling
'  long pollbut
  long channelCount
  
PUB main | tmp, buttons, time, sleeping
  ' buttons holds value of 2 button interface

  lcd.start(lcd#PHILIPS, SCK, SCS, SDIO, SRST) 'using phillips
  'lcd.start(lcd#EPSON, SCK, SCS, SDIO, SRST)

  fds.start(31,30,0,115200)
  eep.start(EEPROM1_SCL, EEPROM1_DAT)
  rem.start(IRIN, @irbuf, rem#IRBUFLEN)
  
  waitcnt(clkfreq/4+cnt)  
  lcd.clear(lcd#WHITE)
  lcd.setContrast($3c)

  scog := cognew(splashScreen, @stack) + 1

  initDatabase                  ' startup database

  key.start(@irbytes)           ' setup key handler
  screen := LIST

  time := clkfreq*5 + CNT
                             
  repeat
{
    if CNT > time
      lcd.sleep
      sleeping := true
}                                            
    repeat
      debug
      buttons := getButtons
      if state == NORMAL and buttons == 1
        repeat 11   ' 11*.03 = .330 ... waits about 1/3 second
          waitcnt(clkfreq/30+cnt) ' check both pressed
          ifnot getButtons  ' both buttons pressed
            buttons~
            btSel.Click
            btOK.Click
            quit
    while buttons == NO_BUTTONS

{
    time := clkfreq*5 + CNT
 
    if sleeping
      lcd.wakeup
      sleeping := false
 }   
    ifnot buttons               ' both buttons pressed
      state := SETTINGS
      screen:= LIST
      showSettingsMode(0)       ' initial menu selected 

    if state == SPLASH or state == INITOK
      case buttons
        1:
          cogstop(scog~ -1)
          btSel.Click
          state := SETTINGS
          showSettingsMode(0)
          'polling~~
          'scog := cognew(pollButtons(@pollbut), @stack) + 1
        2:
          cogstop(scog~ -1)
          btOk.Click
          state := NORMAL
          showNormalMode(0)
          'polling~~
          'scog := cognew(pollButtons(@pollbut), @stack) + 1
        
    elseif state ==  NORMAL
        case buttons
          1:                    ' left  button pressed
            btSel.Click
            selectChannel
          2:                    ' right button pressed
            btOK.Click
            okChannel
                
    elseif state == SETTINGS
        case buttons
          1:                    ' left  button pressed
            btSel.Click
            case screen
              LIST:
                selectSetting
              ADDCH:
                selectEditField(channelCount)
              DELETE:
                selectChannel
              EDIT:
                selectChannel
              EDIT2:
                selectEditField(nmIndex)
              MOVE:
                selectChannel
              MOVE2:
                selectChannel
              
          2:                    ' right button pressed
            btOK.Click
            case screen
              LIST:
                case lbListSm.getIndex
                 SET_ADD:       screen := ADDCH
                    showAddChannel
                 SET_DEL:       screen := DELETE
                    showDeleteChannel
                 SET_EDIT:      screen := EDIT
                    showEditChannelStep1
                 SET_MOVE:      screen := MOVE
                    showMoveChannelStep1
                 SET_NORM:      screen := LIST
                    state := NORMAL
                    showNormalMode(AUTONDX)
                 SET_TRAIN:     screen := TRAIN
                    showTrainDevice
                 SET_TEST:      screen := TTEST
                    showTestDevice
              ADDCH:
                okAddChannel
                showSettingsMode(AUTONDX)
                screen := LIST
              DELETE:
                okDeleteChannel
                showSettingsMode(AUTONDX)
                screen := LIST
              EDIT:
                showEditChannelStep2
                screen := EDIT2
              EDIT2:
                okEditChannel
                showSettingsMode(AUTONDX)
                screen := LIST
              MOVE:
                showMoveChannelStep2
                screen := MOVE2
              MOVE2:
                okMoveChannel
                showSettingsMode(AUTONDX)
                screen := LIST
              TRAIN:
                showSettingsMode(AUTONDX)
                screen := LIST
              TTEST:
                showSettingsMode(AUTONDX)
                screen := LIST
         
PUB getButtons
'  ifnot polling
    return ((ina & (NO_BUTTONS << LBUT)) >> LBUT) & NO_BUTTONS
'  return pollbut
{
PUB pollButtons(butptr)
  repeat
    long[butptr] := ((ina & (NO_BUTTONS << LBUT)) >> LBUT) & NO_BUTTONS
}
            
CON

  #0, SPLASH, INITOK, NORMAL, SETTINGS
  #0, LIST, ADDCH, DELETE, EDIT, EDIT2, MOVE, MOVE2, TRAIN, TTEST, LAST
  #0, SET_TRAIN, SET_TEST, SET_ADD, SET_DEL, SET_EDIT, SET_MOVE, SET_NORM

  SPLASH_FG     = lcd#YELLOW
  SPLASH_BG     = lcd#NAVY '| $17
  SPLASH_BORDER = lcd#CYAN
  SPLASH_FG_COPY= lcd#ORANGE
  
PUB splashScreen

  state := SPLASH

  lcd.clear(SPLASH_BG)
  lcd.setbgcolor(lcd#TRANSPARENT)
  lcd.setfgcolor(SPLASH_FG)

  ' printf overhead is not adding much value here ... change later
  printf(string(CR,{
    }"      TV Favorites  "),0)
  lcd.setfgcolor(SPLASH_FG_COPY)
  printf(string(CR,CR,CR,CR,{
    }"  Copyright (c)  2009",CR,{
    }"     Steve Denson",CR,{
    }"  All Rights Reserved"),0)

  lcd.setfgcolor(SPLASH_FG)
  printf(string(CR,CR,{
    }"Press Setup or OK. Later",CR,{
    }" Select and OK to setup."),0)
    
  showStartupButtons

  splashAnimation
   
PUB splashAnimation | mx, my, n, strn, slen, cmask, smod, fy[30]
  my := 3            
  strn := string("  By Jazzed Design ")
  slen := strsize(strn)
   
  if lcd#MAX_COLORS > $fff
    n := $fff
    cmask := %01110_011111_10011
  else
    n := constant(lcd#PINK+1)
    cmask := $333

  smod := 3
  repeat
    repeat mx from 0 to slen-1
      fy[mx] := n?                
      lcd.dispChar(byte[strn+mx], n | cmask, SPLASH_BG,{
        }mx*lcd#FONTWIDTH+constant(3*lcd#FONTWIDTH), (fy[mx])//smod+my*lcd#FONTHEIGHT+my)
    waitcnt(clkfreq/7+cnt)
    repeat mx from 0 to slen-1
      lcd.dispChar(" ", SPLASH_BG, SPLASH_BG,{
        }mx*lcd#FONTWIDTH+constant(3*lcd#FONTWIDTH), (fy[mx])//smod+my*lcd#FONTHEIGHT+my)
     
CON
  DBSIZE   = CHANNELS
  DBTITLEN = def#TITLE_LEN
  DBCHALEN = def#CHANNEL_LEN
  DBCOOKIE = $CAFEBABE

DAT
  begmark     long $55555555
  cookie      long 0
  cmark       byte "DATA"
  chnum       byte 0 [DBSIZE*DBCHALEN+DBSIZE]   ' up to 4 digits per channel number, make more room later
  title       byte 0 [DBSIZE*DBTITLEN]          ' channel title 
  kmark       byte "KEYS"
  fill        long 0,0,0
  irbytes     byte 0 [IRKEYS*def#IRBUF_LEN]
  dacrc       long 0
  endmark     long $2A2A2A2A

PUB getChannelPtr(n)
  return @chnum+n*def#CHANNEL_LEN

PUB getTitlePtr(n)
  return @title+n*DBTITLEN
         
PUB initDatabase
  loadDatabase
  'dumpBytes(@begmark,getDatabaseLen)
  state := INITOK

PUB clearDatabase
  channelCount~   
  bytefill(@title,0,DBTITLEN*7)
  cookie := DBCOOKIE
  dacrc  := makeCRC
{
  bytemove(getTitlePtr(0),@ts0,strsize(@ts0))
  bytemove(getTitlePtr(1),@ts1,strsize(@ts1))
  bytemove(getTitlePtr(2),@ts2,strsize(@ts2))
  bytemove(getTitlePtr(3),@ts3,strsize(@ts3))
  bytemove(getTitlePtr(4),@ts4,strsize(@ts4))
  bytemove(getTitlePtr(5),@ts5,strsize(@ts5))
  bytemove(getTitlePtr(6),@ts6,strsize(@ts6))

DAT
  ts0          byte "E! Entertainment",0
  ts1          byte "History Channel",0
  ts2          byte "KRON-4 in HD",0
  ts3          byte "The Travel Channel",0
  ts4          byte "The Learning Channel",0
  ts5          byte "The Disney Channel",0
  ts6          byte "CNBC Bloomberg",0
}

PUB loadDatabase | n, rc, mcrc, length, addr
  ' load database here ... do crc check. if invalid crc, blank the memory?
  length := getDatabaseLen
  addr := @begmark
  dprintf(string(CR,"Loading EEPROM $%x @$%x"),p(length)+p(addr))

  eep.Initialize
  repeat n from 0 to length-1 step eep#PageSize
    rc := eep.ReadPage(eep#EEPROM, addr, addr, eep#PageSize)
    if rc
      dprintf(string(CR,"Read Error $d @$%4x"),p(rc)+p(addr))
      snprintf(@b,BLEN,string(CR,"Read Error $d @$%x"),p(rc)+p(addr))
      lcd.drawString(@b,4,4)
      waitcnt(clkfreq+cnt)
    addr += eep#PageSize
    
  channelCount~          
  if cookie == DBCOOKIE and checkCRC
    repeat n from 0 to DBSIZE-1
      if strsize(getTitlePtr(n))
        channelCount++
    return 1
  return 0

PUB saveDatabase | n, rc, mcrc, length, addr
  ' run through channel objects and write to eeprom
  length := getDatabaseLen
  addr := @begmark
  dprintf(string(CR,"Writing EEPROM $%x @$%4x"),p(length)+p(addr))

  cookie := DBCOOKIE
  dacrc  := makeCRC

  ' for some reason page call fails, but long call is fine
  eep.Initialize
  repeat n from 0 to length-1 step 4
    rc := eep.WriteLong(eep#EEPROM, addr, long[addr])
    repeat while eep.WriteWait(eep#EEPROM, addr)
    if rc
      dprintf(string(CR,"Write Error $d @$%4x"),p(rc)+p(addr))
      snprintf(@b,BLEN,string(CR,"Write Error $d @$%4x"),p(rc)+p(addr))
      tbPrompt.init(@b,2,constant(lcd#END_PAGE-45),constant(lcd#END_COL-3),0)
      tbPrompt.setBgColor(lcd#YELLOW)
      tbPrompt.setFgColor(lcd#RED)
      tbPrompt.setText(@b)
      tbPrompt.paint
      waitcnt(clkfreq+cnt)
      return rc
    addr += 4

    
obj        
  crc: "CRC16"
  
PUB getCRC
  return dacrc

PUB makeCRC
  return crc.get(@begmark, getDatabaseLen-4, crc#CRC_INIT)

PUB checkCRC | mcrc
  mcrc := makeCRC
  'dprintf(string("CRC Got $%04x Exp $%4x",CR),p(dacrc)+p(mcrc))
  return mcrc == dacrc

PUB getDatabaseLen
  return @dacrc-@begmark+4
  
PUB saveIrData | n

  tbPrompt.init(string("Saving Remote Data ..."),2,constant(lcd#END_PAGE-45),constant(lcd#END_COL-3),0)
  tbPrompt.setBgColor(lcd#WHITEST_GRAY)
  tbPrompt.paint

  saveDatabase


PUB getIRbytePtr(n)
  return @irbytes+n*def#IRBUF_LEN

  
con
  BT_TOP = lcd#END_PAGE-21
  BG_COLOR = lcd#CYAN

  AUTONDX = 1

DAT ' string
  okText      byte  "   OK ",0
  selectText  byte  " Select", 0
  decFmt      byte  "%d",0
          
PUB showDefaultButtons
  showSelectButton
  showOkButton(@okText)
  
PUB showCancelButton
  btSel.init(string(" Cancel"), 3, BT_TOP, constant(lcd#END_COL/2-12), 0)
  btSel.paint

PUB showStartupButtons
  btSel.init(string(" Setup"), 3, BT_TOP, constant(lcd#END_COL/2-12), 0)
  btSel.paint
  showOkButton(@okText)  

PUB showSelectButton
  btSel.init(@selectText, 3, BT_TOP, constant(lcd#END_COL/2-12), 0)
  btSel.paint
  
PUB showOkButton(caption)
  btOk.init(caption, constant(lcd#END_COL/2+12), BT_TOP, constant(lcd#END_COL/2-14), 0)
  btOk.paint

PUB showDebug(txtptr)
  tbDebug.init(txtptr,2,constant(lcd#END_PAGE-40),constant(lcd#END_COL-3),0)
  tbDebug.setBgColor(lcd#WHITEST_GRAY)
  tbDebug.paint

CON


var
  byte nmIndex, smIndex
    
PUB showNormalMode(ndx)
  lcd.clear(BG_COLOR)

  showChannelList(ndx)
  showDefaultButtons

PUB selectChannel
  nmIndex := lbListUm.getIndex+1 
  lbListUm.setIndex(nmIndex // (channelCount+1))  
  lbListUm.change

PUB okChannel
  ' send channel to TV

PUB showChannelList(ndx) | k

  lbListUm.init(7,2,1,constant(lcd#END_COL-3),constant(lcd#END_PAGE-25))
  lbListUm.setBorderColor(lcd#WHITEST_GRAY)
  lbListUm.setBorderColor(BG_COLOR)
  lbListUm.setGap(1)
  repeat k from 0 to CHANNELS 
    lbListUm.setIndex(k)
    lbListUm.setText(getTitlePtr(k))
  ifnot ndx
    lbListUm.setIndex(ndx)
    nmIndex~
  else
    lbListUm.setIndex(nmIndex)    
  lbListUm.paint


CON
  
PUB showSettingsMode(ndx)
  lcd.clear(BG_COLOR)

  lbListSm.init(7,2,1,constant(lcd#END_COL-3),constant(lcd#END_PAGE-24))
  lbListSm.setBorderColor(lcd#WHITEST_GRAY)
  lbListSm.setBorderColor(BG_COLOR)
  lbListSm.setGap(1)
  lbListSm.setIndex(SET_TRAIN)
  lbListSm.setText(string("Train Device"))
  lbListSm.setIndex(SET_TEST)
  lbListSm.setText(string("Test Device"))
  lbListSm.setIndex(SET_ADD)
  lbListSm.setText(string("Add Channel"))
  lbListSm.setIndex(SET_DEL)
  lbListSm.setText(string("Delete Channel"))
  lbListSm.setIndex(SET_EDIT)
  lbListSm.setText(string("Edit Channel"))
  lbListSm.setIndex(SET_MOVE)
  lbListSm.setText(string("Move Channel"))
  lbListSm.setIndex(SET_NORM)
  lbListSm.setText(string("Show TvFavorites"))
  ifnot ndx
    lbListSm.setIndex(ndx)
    smIndex~
  else                       
    lbListSm.setIndex(smIndex)
  lbListSm.paint

  showDefaultButtons

PUB selectSetting
  smIndex := lbListSm.getIndex+1 
  lbListSm.setIndex(smIndex // lbListSm.getCount)  
  lbListSm.change

  
VAR
  byte tbIndex
  
CON

PUB showEdit(chan) | cs, clen, ts, tlen

  ts := string("Channel Name:")
  tlen := strsize(ts)
  laInput[tbIndex].init(ts,4,tbStatus.getTop+tbStatus.getHeight+12,tlen*lcd#FONTWIDTH,0)
  laInput[tbIndex].paint
  tbInput[tbIndex].init(getTitlePtr(channelCount),2,laInput[tbIndex].getTop+laInput[tbIndex].getHeight+4,constant(lcd#END_COL-3),0)
  tbInput[tbIndex].paint

  cs := string("Channel Number:")
  clen := strsize(cs)
  laInput[tbIndex+1].init(cs,4,tbInput[tbIndex].getTop+tbInput[tbIndex].getHeight+8,clen*lcd#FONTWIDTH-4,0)
  laInput[tbIndex+1].paint

  tbInput[tbIndex+1].init(getChannelPtr(channelCount), laInput[tbIndex+1].getWidth+constant(2*lcd#FONTWIDTH),laInput[tbIndex+1].getTop,lcd#END_COL-laInput[tbIndex+1].getWidth-8-3,0)
  tbInput[tbIndex+1].paint

  showEditKeys(string("keys "))
  selectEditField(chan)

PUB selectEditField(chan) | tp, k, n, ch, oc, len
  'dprintf(string("Enter selectAddField %d.",$d),p(tbIndex))           
  n~                                                    ' start with first char
  tbInput[tbIndex].setHighlight(lcd#YELLOW)
  tbInput[tbIndex].setCursor(n)
  ifnot tbIndex                                         ' set correct field text
    tp := getTitlePtr(chan)
    len:=def#TITLE_LEN-1
  else                 
    tp := getChannelPtr(chan)                   
    len:=def#CHANNEL_LEN-1
  ifnot byte[tp]                                        ' make sure we don't display garbage
    byte[tp] := " "
  tbInput[tbIndex].setText(tp)
  tbInput[tbIndex].paint
  waitcnt(clkfreq/2+cnt)                                ' debounce buttons
  repeat while getButtons == NO_BUTTONS
    k := key.getKeyCode
    if k > -1
      case k
        key#LEFT:                                       ' left arrow
          if n > 0
            n--
            ch := " "
        key#RIGHT:
          if n < len                                    ' right arrow
            n++
        other:
          ch := key.getKey(k)                           ' a char
          showEditKeys(key.getKeyString(k))             ' key reminder
          tbInput[tbIndex].setCursor(n)
          tbInput[tbIndex].setCharAt(n,ch)              ' save char to pointer

    ch := tbInput[tbIndex].getCharAt(n)                 ' char for capitalization
    if ia.isalpha(ch)                                   ' only edit letters
      if n > 0                                          
        oc := tbInput[tbIndex].getCharAt(n-1)
        if oc == " "
          ch &= !$20
      else
        ch &= !$20
    tbInput[tbIndex].setCursor(n)                       ' show new char
    tbInput[tbIndex].setCharAt(n,ch)
    tbInput[tbIndex].paint

  tbInput[tbIndex].setCursor(-1)                        ' shut off cursor
  tbInput[tbIndex].paint
  tbIndex := (++tbIndex) & 1                            ' set next textbox
  waitcnt(clkfreq/2+cnt)                                ' debounce buttons

PUB showEditKeys(str) | n
  tbDebug.init(str, tbInput[1].getLeft,laInput[0].getTop,tbInput[1].getWidth,0)
  tbDebug.setBgColor(lcd#WHITEST_GRAY)
  tbDebug.setCursor(key.getClick)
  tbDebug.paint

CON

PUB showAddChannel
  lcd.clear(BG_COLOR)
  
  tbStatus.init(string(" Add a Favorite Channel"),2,4,constant(lcd#END_COL-3),0)
  tbStatus.setBgColor(lcd#WHITEST_GRAY)
  tbStatus.paint

  showDefaultButtons

  tbIndex~
  showEdit(channelCount)
  
PUB okAddChannel | count, n
  count~
  if cookie == DBCOOKIE and checkCRC
    repeat n from 0 to DBSIZE-1
      if strsize(getTitlePtr(n))
        count++
  channelCount := count
  ifnot checkCRC                ' only save if CRC is different
    saveDatabase
  
CON

PUB showDeleteChannel
  lcd.clear(BG_COLOR)

  showChannelList(0)
  
  showSelectButton
  showOkButton(string(" Delete"))

PUB selectDelete
  selectChannel

PUB okDeleteChannel | n
  ' need to copy channels delch+1 to channelCount-1 to delch space+
  ' then channelCount--
  repeat n from nmIndex to channelCount-1
    bytemove(getTitlePtr(n),getTitlePtr(n+1),strsize(getTitlePtr(n)))
    bytemove(getChannelPtr(n),getChannelPtr(n+1),strsize(getChannelPtr(n)))
  channelCount--
  ifnot checkCRC                ' only save if CRC is different
    'saveDatabase

CON

PUB showEditChannelStep1
  lcd.clear(BG_COLOR)

  showChannelList(0)
  
  showSelectButton
  showOkButton(string("  Edit"))

PUB showEditChannelStep2 | cs, clen, ts, tlen
  lcd.clear(BG_COLOR)
  
  tbStatus.init(string(" Edit Favorite Channel"),2,4,constant(lcd#END_COL-3),0)
  tbStatus.setBgColor(lcd#WHITEST_GRAY)
  tbStatus.paint

  showDefaultButtons

  tbIndex~
  showEdit(nmIndex)

PUB selectEdit
  selectChannel

PUB okEditChannel
  ifnot checkCRC                ' only save if CRC is different
    saveDatabase

CON

VAR
  long fromIndex
  byte tmpTitle[DBTITLEN]
  byte tmpChan[DBCHALEN]
  
PUB showMoveChannelStep1
  lcd.clear(BG_COLOR)

  showChannelList(AUTONDX)
  showSelectButton
  showOkButton(string(" Choose"))

PUB showMoveChannelStep2
  lcd.clear(BG_COLOR)
  fromIndex := nmIndex // (channelCount+1)
  showChannelList(0)
  showDefaultButtons

PUB selectMove
  selectChannel

PUB okMoveChannel | n, m, toIndex
  toIndex := nmIndex
  if fromIndex == toIndex
    return                      ' no move
  bytemove(@tmpTitle,getTitlePtr(fromIndex),DBTITLEN)
  bytemove(@tmpChan,getChannelPtr(fromIndex),DBCHALEN)
  dprintf(string($d,"Move %d '%s' '%s'",$d),p(fromIndex)+p(@tmpChan)+p(@tmpTitle))           
  ' if toIndex < fromIndex  
  '   copy fromIndex item into tmp
  '   copy any item to toIndex from item toIndex-1
  '   or copy any item to fromIndex from item fromIndex-1
  '   copy tmp to toIndex
  ' if nmIndex > fromIndex
  '   copy fromIndex item into tmp
  '   copy any item from fromIndex+1 to item fromIndex
  '   copy tmp to toIndex
  if toIndex < fromIndex
{ can't do it this way ... broken spin    
    repeat n from fromIndex to toIndex step-1
      'dprintf(string("Move %d <- %d.",$d),p(n)+p(n-1))           
      bytemove(getTitlePtr(n),getTitlePtr(n-1),DBTITLEN)
      bytemove(getChannelPtr(n),getChannelPtr(n-1),DBCHALEN)
}      
    n := fromIndex
    repeat
      bytemove(getTitlePtr(n),getTitlePtr(n-1),DBTITLEN)
      bytemove(getChannelPtr(n),getChannelPtr(n-1),DBCHALEN)
    while n-- > toIndex
  else
    repeat n from fromIndex to toIndex-1
      'dprintf(string("Move %d <- %d.",$d),p(n)+p(n+1))           
      bytemove(getTitlePtr(n),getTitlePtr(n+1),DBTITLEN)
      bytemove(getChannelPtr(n),getChannelPtr(n+1),DBCHALEN)
      
  dprintf(string("Saving %d '%s' '%s'",$d),p(toIndex)+p(@tmpChan)+p(@tmpTitle))           
  bytemove(getTitlePtr(toIndex),@tmpTitle,DBTITLEN)
  bytemove(getChannelPtr(toIndex),@tmpChan,DBCHALEN)
  ifnot checkCRC                ' only save if CRC is different
    saveDatabase

con
  KEY_LEFT = 25
  KEY_TOP  = 25
  KEY_WIDTH = lcd#FONTWIDTH*4

PUB isKeyCode | k, rc
  repeat k from 0 to IRKEYS-1
    rc := rem.match(getIRbytePtr(k),0)
    ifnot rc
      return 1
  return 0

PUB showTrainDevice | k, keys[IRKEYS], phase, rc, rr
{
[TvFavorites Training]
  [1] [2] [3] [4]
  [5] [6] [7] [8]
  [9] [<] [0] [>]
    [enter]
[Press remote 1 key]
}
  lcd.clear(BG_COLOR)

  tbStatus.init(string(" "),2,4,lcd#END_COL-3,0)
  tbStatus.setBgColor(lcd#WHITEST_GRAY)

  repeat k from 0 to 9
    ia.itoa((k+1)//10,@keys[k])

  bytemove(@keys[10],string("<"),2)
  bytemove(@keys[11],string(">"),2)
  repeat k from 0 to 11
    btKey[k].init(@keys[k], KEY_LEFT+KEY_WIDTH*(k//4), KEY_TOP+20*(k/4), constant(KEY_WIDTH-2), 0)
    btKey[k].paint

  showSelectButton
  showOkButton(@okText)
  
  btSel.setFgColor(lcd#LIGHT_GRAY)
  btSel.paint
  btOK.setFgColor(lcd#LIGHT_GRAY)
  btOK.paint
  
  tbPrompt.init(@b,2,constant(lcd#END_PAGE-45),constant(lcd#END_COL-3),0)
  tbPrompt.setBgColor(lcd#WHITEST_GRAY)

  tbDebug.init(string(" "),constant(lcd#END_COL/2-7),constant(BT_TOP+2),18,0)
  tbDebug.paint
    
  repeat k from 0 to IRKEYS-1

    repeat phase from 0 to 2    ' 0 is acquire, 1 is test
    
      case k
        10:
          snprintf(@b,BLEN,string("Remote Key for Back"),0)
        11:
          snprintf(@b,BLEN,string("Remote Key for Next"),0)
        other:
          snprintf(@b,BLEN,string(" Press Remote '%s' Key"),p(btKey[k].getText))
     
      tbPrompt.setText(@b)     
      tbPrompt.paint
     
      repeat ' while not valid

        ifnot phase
          tbStatus.setText(string(" TvFavorites Training"))
        else
          tbStatus.setText(string(" TvFavorites Testing"))
        tbStatus.paint

        tbDebug.setText(string(" "))
        tbDebug.setBgColor(lcd#WHITE)
        tbDebug.paint

        ifnot phase ' acquire phase
          waitcnt(clkfreq/10+cnt)
          rc := rem.acquire(500)
          if isKeyCode
            next
            
          bytemove(getIRbytePtr(k),@irbuf,rem#IRBUFLEN)
          snprintf(@b,BLEN,@decFmt,p(rc))
          
        else        ' test phase
        
          repeat
            rc := rem.acquire(500)
          while rc

          rr := rem.match(getIRbytePtr(k),0)
          if rr ' if mismatch, restart training
          
            tbDebug.setText(string("?"))
            tbDebug.paint
            waitcnt(clkfreq/2+cnt)
            if phase > 1
              phase~
            else
              phase++
            waitcnt(clkfreq/2+cnt)
            next

          else
            btKey[k].set3dmode(false)
            snprintf(@b,BLEN,@decFmt,p(rc))
            phase := 3
                
        ifnot rc
          tbDebug.setText(string(" "))
          tbDebug.setBgColor(lcd#CYAN)
          tbDebug.paint
          quit

        waitcnt(clkfreq/10+cnt)

      if phase     

        btKey[k].setDefault(false)
        btKey[k].paint
         
        if k < IRKEYS-1
          btKey[k+1].setDefault(true)
          btKey[k+1].paint         

  showTestDevice

PUB showTestDevice | k, rc, keys[IRKEYS], done
{
[TvFavorites Training]
  [1] [2] [3] [4]
  [5] [6] [7] [8]
  [9] [<] [0] [>]
    [enter]
[Press remote 1 key]
}
  lcd.clear(BG_COLOR)

  tbStatus.init(string(" TvFavorites Key Test"),2,4,constant(lcd#END_COL-3),0)
  tbStatus.setBgColor(lcd#WHITEST_GRAY)
  tbStatus.paint

  tbDebug.init(string(" "),constant(lcd#END_COL/2-7),constant(BT_TOP+2),18,0)
  tbDebug.paint
             
  repeat k from 0 to 9
    ia.itoa((k+1)//10,@keys[k])
    
  bytemove(@keys[10],string("<"),2)
  bytemove(@keys[11],string(">"),2)
  repeat k from 0 to 11
    btKey[k].init(@keys[k], KEY_LEFT+KEY_WIDTH*(k//4), KEY_TOP+20*(k/4), constant(KEY_WIDTH-2), 0)
    btKey[k].paint

  tbPrompt.init(string("Click All Keys to Finish"),2,constant(lcd#END_PAGE-45),constant(lcd#END_COL-3),0)
  tbPrompt.setBgColor(lcd#WHITEST_GRAY)
  tbPrompt.paint

  showCancelButton
  showOkButton(@okText)
  btOK.setFgColor(lcd#LIGHT_GRAY)
  btOK.paint

  repeat k from 0 to IRKEYS-1
    keys[k]~

  repeat 
    ' check key score to finish
    done~
    repeat k from 0 to IRKEYS-1
      if keys[k]
        done++

    snprintf(@b,BLEN,@decFmt,p(done))        
    tbDebug.setText(@b)
    tbDebug.paint
     
    if done > IRKEYS-1
      quit
      
    if getButtons == 1
      screen := LIST
      btSel.Click
      showSettingsMode(0)
      quit

    rc := rem.acquire(500)

    ifnot rc
      repeat k from 0 to IRKEYS-1
        
        rc := rem.match(getIRbytePtr(k),0)
        snprintf(@b,BLEN,@decFmt,p(k))        
        ifnot rc
          btKey[k].Click
          keys[k] := " "
          quit
         
      if k => IRKEYS            
        tbPrompt.setText(string(" No Key Match Found"))
      else
        tbPrompt.setText(string(" Button Key Matched"))
      tbPrompt.paint

  btOK.setFgColor(lcd#BLACK)
  btOK.paint

  repeat
    case getButtons
        1: screen := LIST
           btSel.Click
          showSettingsMode(0)
          quit
        2: screen := LIST
          btOK.Click
          saveIrData
          showSettingsMode(0)
          quit
      

con '' support for printf
  BLEN = 80

obj
  fds : "FullDuplexSerialAPI" ' turn off output with "FullDuplexSerialAPI"
  
var
  byte b[BLEN+1]
  
PUB printf(fmt, args)
  snprintf(@b,BLEN,fmt,args)
  lcd.str(@b)

PUB p(v)
  sf.p(v)

PUB snprintf(bp,len,fmt,args)
  sf.snprintf(bp,len,fmt,args)

PUB dprintf(fmt, args)
  snprintf(@b,BLEN,fmt,args)
  fds.str(@b)

PUB debug | ch, len
{
  len := getDatabaseLen 
  ch := fds.rxcheck
  case ch
    "d":
      dumpBytes(@begmark,len+4)
    "l":
      loadDatabase
    "s":
      saveDatabase
    "i":
      clearDatabase
    "c":
      dacrc := makeCRC
      dprintf(string(CR,"DB @$%04x $%x bytes. CRC @$%04x = $%04x",CR),p(@begmark)+p(len)+p(@dacrc)+p(dacrc))
    "g":
      dprintf(string(CR,"DB @$%04x $%x bytes. CRC @$%04x = $%04x",CR),p(@begmark)+p(len)+p(@dacrc)+p(getCRC))
      
PUB dumpBytes(dptr, length) | n, ii, bytes[17]
  dprintf(string($d),0)
  repeat ii from 0 to length-1
    ifnot ii // 16
      dprintf(string($d,"%04x "),p(dptr+ii))
    dprintf(string("%02x"),p(byte[dptr+ii]))
    bytes[ii // 16] := byte[dptr+ii] 
    ifnot ii // 16 == 15
      dprintf(string(" "),0)
    else
      dprintf(string(" "),0)
      repeat n from 0 to 15
        if bytes[n] => $20 and bytes[n] < $7f
          dprintf(string("%c"),p(bytes[n]))
        else
          dprintf(string("."),0)
  if ii
      dprintf(string(" "),0)
      repeat n from 0 to 15
        if bytes[n] => $20 and bytes[n] < $7f
          dprintf(string("%c"),p(bytes[n]))
        else
          dprintf(string("."),0)
  dprintf(string($d),0)
'}

{{
                            TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}                                         