{{
TvKeys.spin: - Defines a TV remote phone-like key interface
Copyright (c) 2009 John Steven Denson
See MIT license terms below
}}
  LEFT      = 10 ' always back up ... no erase or delete
  RIGHT     = 11 ' next char
  CHARS     = 5  ' number of chars on a key


{{
Test Code
}}
{
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  SCS   = 27
  SCK   = 26
  SDIO  = 25
  SRST  = 24

  IRIN  = 16

obj
  d  : "spiasm-lcd-65k"
  sf : "snprintf"

var
  byte irbuf[def#IRBUF_LEN+1]
  
PUB main | n, mx, my, slen, strn, r

  rem.start(IRIN, @irbuf, def#IRBUF_LEN)
  start(@irbuf)
  
  d.start(d#PHILIPS, SCK, SCS, SDIO, SRST)
  
  d.clear(d#BLUE)
  d.setfgcolor(d#YELLOW)
  d.setbgcolor(d#TRANSPARENT)
  printf(string($d," IR Key Tester",$d),0)

  n~
  repeat
    n := getKeyCode
    if n > 0
      printf(string("Got Key %c",$d),p(getKey(n)))
    
  repeat

con '' support for printf
  BLEN = $80
  
var
  byte b[BLEN+1]
  
PUB printf(fmt, args)
  snprintf(@b,BLEN,fmt,args)
  fds.str(@b)

PUB p(v)
  sf.p(v)

PUB snprintf(bp,len,fmt,args)
  sf.snprintf(bp,len,fmt,args)

PUB dprintf(fmt, args)
  snprintf(@b,BLEN,fmt,args)
  fds.str(@b)
  sf : "snprintf"
  fds: "FullDuplexSerialDebug" ' "FullDuplexSerialAPI" ' empty version
'}
    
obj
  def: "TvDefines"
  rem: "TvRemote"

var
  byte click          ' save click count for a key
  byte last           ' save last key
  word irbp           ' irbyte array pointer
    
pub start(ptr) | len
{{
  initialize object
  @param ptr - pointer to IR byte array
}}
  irbp := ptr
  len  := constant((def#IRKEYS-1)*def#IRBUF_LEN)
  click:= CHARS-1            


pub getIRbytePtr(n)
  return irbp+n*def#IRBUF_LEN

pub getKeyCode : rc | k
{{
  Wait for a key, lookup key number based on match.
  Slow, but reasonable procedure given small number of keys.
  @return -1 if no key or key number
}}
  rc := rem.acquire(1000)             ' acquire has fixed timeout ... should be settable
  ifnot rc
    repeat k from 0 to constant(def#IRKEYS-1)
      rc := rem.match(getIRbytePtr(k),0)
      ifnot rc
        quit
    if k > def#IRKEYS-1            
      rc := -1
    else
      rc := k
      if last == rc
        click := ++click // CHARS
      else
        click~
      last := rc
  else
    return -1
  return rc

pub getKey(v) : rc | ptr
{{
  Use a "key map" <number> << 4 | click count to translate key clicks to ASCII
  @param  v - key returned from getKey call
  @return "current" key ... that is key pressed + number of times key is pressed
}}      
  if v < 0
    result := 0
  case v
    LEFT:   return LEFT
    RIGHT:  return RIGHT
    other:
      ptr := @one+v*constant(CHARS+1)+click 
      return byte[ptr]

pub getClick
  return click
      
DAT
  one   byte "1-+/!",0
  two   byte "2abc@",0
  three byte "3def#",0
  four  byte "4ghi$",0
  five  byte "5jkl%",0
  six   byte "6mno^",0
  seven byte "7pqrs",0
  eight byte "8tuv*",0
  nine  byte "9wxyz",0
  zero  byte "0 '&=",0
  null  byte 0

pub getKeyString(k)
{{
  Get key chars available on a key.
  @param key - key number
  @return key string pointer or "null string" if invalid key
}}
  if k > -1 and k < 12
    return @one+k*constant(CHARS+1)
  return @null 

pub rx : rc
{{
  Use a "key map" <number> << 4 | click count to translate key clicks to ASCII
  @return "current" key ... that is key pressed + number of times key is pressed
}}
  rc := getKeyCode
  case rc
    0:  return 0
    11: return LEFT
    12: return RIGHT
    other: return byte[@one+rc*constant(CHARS+1)+click]

{{
                            TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
            