{{
gui_listbox.spin:
- Defines an LCD listbox
Copyright (c) 2009 John Steven Denson
See MIT license terms below
}}

var
  long textptr[g#LISTBOX_LEN]
  word count                    '' the number of boxes to display
  word index                    '' the index of box for getText/setText
  word length                   '' the number of boxes with text
  word left                     '' left pixel position of element
  word top                      '' top pixel position of element
  word width                    '' pixel width of element -- set to 0 for default or auto size
  word height                   '' pixel height of element -- set to 0 for default or auto size
  word gap                      '' vertical gap between boxes
  word bgcolor                  '' bgcolor for listbox
  word fgcolor                  '' fgcolor for listbox
  word selcolor                 '' selected entry bgcolor
  word bordercolor              '' color to use for border
  
obj d: "spiasm-lcd-65k"
    g: "gui_common"
  
pub init(c,x,y,w,h)
{{
  listbox constructor - creates a proportionally spaced list of count text boxes in given width/height
  @param c  - number of text boxes to display
  @param x  - left
  @param y  - top
  @param w  - width  use 0 for default size
  @param h  - height use 0 for default size
}}
  count := c
  index := -1
  left  := x
  top   := y
  width := w
  height:= h
  gap   := 0
  setbgcolor(d#WHITE)
  setfgcolor(d#BLACK)
  setselcolor(d#WHITEST_GRAY)
  setbordercolor(d#WHITE_GRAY)
  
pub setCount(v)
  count := v

pub getCount
  return count

pub setIndex(v)
  index := v

pub getIndex
  return index

pub setSelColor(v)
  selcolor := v

pub getSelColor
  return selcolor

pub setBorderColor(v)
  bordercolor := v

pub getBorderColor
  return bordercolor

pub setGap(v)
  gap := v

pub getGap
  return gap

pub setText(v)
  textptr[index] := v

pub setTop(v)
  top := v
  
pub setLeft(v)
  left := v

pub setWidth(v)
  width := v

pub setHeight(v)
  top := v

pub setBgColor(v)  
  bgcolor := v  
  
pub setFgColor(v)  
  fgcolor := v  
  
pub getText
  return textptr[index]

pub getTop
  return top
  
pub getLeft
  return left

pub getWidth
  return width

pub getHeight
  return height

pub getBgColor  
  return bgcolor  
  
pub getFgColor  
  return fgcolor  

pub paint | n, h, w, p

  if count =< 0
    return
  p := height*3/d#END_PAGE
  h := height/count
  w := width-2
  d.setbgcolor(bgcolor)
  d.setfgcolor(fgcolor)
  d.fillRectangle(bordercolor,left,top,width,height)
  
  repeat n from 0 to count-1

    d.fillRectangle(bgcolor,1+left,p+top+n*h,w,h-gap)
    d.drawRectangle(d#WHITE_GRAY,1+left,p+top+n*h,w-1,h-gap-1,1)
    d.drawRectangle(d#LIGHT_GRAY,1+left+1,p+top+1+n*h,w-2,h-gap-2,1)
    
    if index => 0 and n == index
      d.setbgcolor(selcolor)
      d.fillRectangle(selcolor,1+left+2,p+top+2+n*h,w-4,h-gap-3)
    else
      d.setbgcolor(bgcolor)
      d.fillRectangle(bgcolor,1+left+2,p+top+2+n*h,w-4,h-gap-3)
      
    if textptr[n]
      d.drawString(textptr[n],left+d#FONTWIDTH,p+top+n*h+h*1/5)
      
    d.drawRectangle(d#WHITEST_GRAY,1+left+1,p+top+2+n*h,w-3,h-gap-3,1)

pub change | n, h, w, p

  if count =< 0
    return
  p := height*3/d#END_PAGE
  h := height/count
  w := width-2
  d.setbgcolor(bgcolor)
  d.setfgcolor(fgcolor)
  
  'dont erase, just change
  'd.fillRectangle(bordercolor,left,top,width,height)
  
  repeat n from index-1 to index

    d.fillRectangle(bgcolor,1+left,p+top+n*h,w,h-gap)
    d.drawRectangle(d#WHITE_GRAY,1+left,p+top+n*h,w-1,h-gap-1,1)
    d.drawRectangle(d#LIGHT_GRAY,1+left+1,p+top+1+n*h,w-2,h-gap-2,1)
    
    if index => 0 and n == index
      d.setbgcolor(selcolor)
      d.fillRectangle(selcolor,1+left+2,p+top+2+n*h,w-4,h-gap-3)
    else
      d.setbgcolor(bgcolor)
      d.fillRectangle(bgcolor,1+left+2,p+top+2+n*h,w-4,h-gap-3)

    if n < 0
      d.setbgcolor(bgcolor)
      d.fillRectangle(bgcolor,1+left+2,p+top+2+(count-1)*h,w-4,h-gap-3)
      d.drawString(textptr[count-1],left+d#FONTWIDTH,p+top+(count-1)*h+h*1/5)
    elseif textptr[n]
      d.drawString(textptr[n],left+d#FONTWIDTH,p+top+n*h+h*1/5)
      
    d.drawRectangle(d#WHITEST_GRAY,1+left+1,p+top+2+n*h,w-3,h-gap-3,1)

{{
                            TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
          